namespace SortControls
{

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Reflection;

class SortButton
  {
  public string Name;
  public int X;
  public int Y;
  public EventHandler Action;
  public Button Button;
  public SortButton(string n, int x0, int y0, EventHandler a)
    {
    Name = n;
    X = x0;
    Y = y0;
    Action = a;
    }
  }

public class SortForm : Form
  {
  public const int xWidth = 150;
  public const int xOffset = 10;

  public const int yHeight = 148;
  public const int yTitle = 24;
  public const int xButtonWidth = 80;
  public const int yButtonHeight = 24;
  public const int yOffset = 10;

  protected internal int active_threads = 0;

  static SortButton[] btn;

  static Type[] types;
  static string[] algs;
  static string[] langs;
  static string[] choices;

  // this is to get the type that inherits from SortPanel
  static Type getPanelType(string name)
    {
    Assembly a = Assembly.Load(name);
    Type[] t = a.GetTypes();
    for (int j = 0; j < t.Length; j++)
      {
      Type x = t[j].BaseType;
      if (x == typeof(SortPanel))
	{
	return t[j];		// return the inheriting type
	}
      }
    throw new ApplicationException(
      "Could not find SortPanel in assembly "+name);
    }

  static SortForm()
    {
    DirectoryInfo di = new DirectoryInfo(".");
    FileInfo[] files = di.GetFiles("Alg*.dll");

    algs = new string[files.Length];
    langs = new string[files.Length];
    choices = new string[files.Length];
    types = new Type[files.Length];

    for (int i = 0; i < files.Length; i++)
      {
      Match m = Regex.Match(files[i].Name, @"(\w+).dll");
      string name = m.Groups[1].ToString();
      types[i] = getPanelType(name);
      SortPanel p = (SortPanel) Activator.CreateInstance(types[i]);
      algs[i] = p.sortAlg;
      langs[i] = p.sortLang;
      choices[i] = langs[i] + ": "+ algs[i];
      }
    }

  void createButtons()
    {
    int y = yOffset*2 + yTitle + yHeight;
    btn = new SortButton[]
    {
      new SortButton("Random",  120, y, new EventHandler(MakeRandom)),
      new SortButton("Reverse", 232, y, new EventHandler(MakeReverse)),
      new SortButton("Start",   344, y, new EventHandler(SortStart)),
      new SortButton("Stop",    456, y, new EventHandler(SortStop))
    };

    foreach (SortButton sb in btn)
      {
      Button b = new Button();
      b.Location = new Point(sb.X, sb.Y);
      b.Size = new Size(xButtonWidth, yButtonHeight);
      b.Text = sb.Name;
      b.Click += sb.Action;
      sb.Button = b;
      Controls.Add(b);
      }
    }

  int[] array = new int[yHeight/2];
  SortPanel[] sdp;
  ComboBox[] sdc;
  int numPanels = 0;

  void createPanels()
    {
    numPanels = 4;		// just create 4 panels for now
    scramble(array);		// scramble the array for use
    sdp = new SortPanel[numPanels];
    sdc = new ComboBox[numPanels];

    for (int i = 0; i < numPanels; i++)
      {
      sdp[i] = createSortPanel(i, xOffset+i*(xWidth+xOffset), yOffset);
      Controls.Add(sdp[i]);

      sdc[i] = new ComboBox();
      sdc[i].DropDownStyle = ComboBoxStyle.DropDownList;
      sdc[i].Items.AddRange(choices);
      sdc[i].Size = new Size(xWidth, yTitle);
      sdc[i].Location = new Point(xOffset+i*(xWidth+xOffset),
				  yHeight+yOffset+yOffset);
      sdc[i].SelectedIndex = i;
      sdc[i].SelectedIndexChanged += new EventHandler(SortSelect);
      Controls.Add(sdc[i]);
      }
    }

  SortPanel createSortPanel(int index, int x, int y)
    {
    SortPanel p = (SortPanel) Activator.CreateInstance(types[index]);
    p.init(this);
    p.setArray(array);
    p.Location = new Point(x, y);
    return p;
    }

  void SortSelect(object o, EventArgs e)
    {
    for (int i = 0; i < numPanels; i++)
      if (sdc[i] == o)
	{
	sdp[i].stopThread();	// stop the current thread
	Controls.Remove(sdp[i]); // remove the current panel from controls
	int index = sdc[i].SelectedIndex; // get new alg index
	sdp[i] = createSortPanel(index, xOffset+i*(xWidth+xOffset), yOffset);
	Controls.Add(sdp[i]);
	return;
	}
    }

  NumericUpDown spinner;
  Label spinLab;
  void createSpinner()
    {
    int x = btn[btn.Length-1].X + xButtonWidth + xOffset*2;
    int y = yOffset*2 + yTitle + yHeight;
    int xLab = 42;
    spinLab = new Label();
    spinLab.Text = "Delay:";
    spinLab.TextAlign = ContentAlignment.MiddleCenter;
    spinLab.Location = new Point(x, y);
    spinLab.Size = new Size(xLab, yButtonHeight);
    Controls.Add(spinLab);
    spinner = new NumericUpDown();
    spinner.Location = new Point(xLab+x, y+2);
    spinner.TextAlign = HorizontalAlignment.Center;
    spinner.ReadOnly = true;
    spinner.Maximum = 9;
    spinner.Minimum = 0;
    spinner.Value = 5;
    spinner.Size = new Size(30, yButtonHeight);
    Controls.Add(spinner);
    spinner.ValueChanged += new EventHandler(SetSpeed);
    }

  internal int sleep = 50;
  void SetSpeed(object o, EventArgs e)
    {
    sleep = ((int) spinner.Value) * 10;
    }

  public SortForm(string t)
    {
    Text = t;

    int xSize = xOffset+4*(xWidth+xOffset);
    int ySize = yOffset*4+yHeight+yButtonHeight+yTitle;
    ClientSize = new Size(xSize, ySize);
    BackColor = Color.Beige;
    FormBorderStyle = FormBorderStyle.Fixed3D;
    createButtons();
    resetButtons();
    createSpinner();
    createPanels();
    }

  void MakeRandom(object sender, EventArgs e)
    {
    // random
    scramble(array);
    for (int i = 0; i < numPanels; i++)
      {
      sdp[i].setArray(array);
      sdp[i].Invalidate();
      }
    }

  void MakeReverse(object sender, EventArgs e)
    {
    // Reverse
    reverse(array);
    for (int i = 0; i < numPanels; i++)
      {
      sdp[i].setArray(array);
      sdp[i].Invalidate();
      }
    }

  void SortStart(object sender, EventArgs e)
    {
    // Start
    btn[0].Button.Enabled = false;
    btn[1].Button.Enabled = false;
    btn[2].Button.Enabled = false;
    btn[3].Button.Enabled = true;
    active_threads = numPanels;

    for (int i = numPanels-1; i >= 0; i--)
      {
      sdp[i].startSort();
      }
    }

  void SortStop(object sender, EventArgs e)
    {
    // Stop
    for (int i = 0; i < numPanels; i++)
      {
      sdp[i].stopSort();
      }
    resetButtons();
    }

  // This is a seperate method so that it may be easily called from sort thread
  protected internal void resetButtons()
    {
    // Stopped state
    btn[0].Button.Enabled = true;
    btn[1].Button.Enabled = true;
    btn[2].Button.Enabled = true;
    btn[3].Button.Enabled = false;
    }

  protected override void OnClosing(CancelEventArgs e)
    {
    for (int i = 0; i < numPanels; i++)
      {
      sdp[i].stopSort();
      sdp[i].stopThread();
      }
    }

  public void Run()
    {
    Application.Run(this);
    }

  /*
   * Fill the array with random numbers from 0..n-1.
   */
  void scramble(int[] a)
    {
    double f = xWidth / (double) a.Length;
    a[0] = -1;
    for (int i = a.Length; --i >= 0;)
      {
      a[i] = (int)(i * f);
      }
    Random r = new Random();
    for (int i = a.Length; --i >= 0;)
      {
      int j = r.Next(i);
      int t = a[i];
      a[i] = a[j];
      a[j] = t;
      }
    }

  /**
    * Fill the array with reverse order
    */
  void reverse(int[] a)
    {
    double f = xWidth / (double) a.Length;
    int N = a.Length;
    a[0] = -1;
    for (int i = 0; i < N; i++)
      {
      a[N-i-1] = (int)(i * f);
      }
    }
  }

}
