namespace SortControls
{

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

public abstract class SortPanel : Panel
  {
  string thr;
  private Thread t;

  int[] array;
  protected internal bool stopRequested = true; // start as true, reset is always required

  SortForm root = null;

  Brush grayBrush = new SolidBrush(Color.Gray);
  Brush whiteBrush = new SolidBrush(Color.White);
  Pen bluePen = new Pen(new SolidBrush(Color.Blue));
  Pen blackPen = new Pen(new SolidBrush(Color.Black));
  Pen redPen = new Pen(new SolidBrush(Color.Red));

  protected internal SortPanel()
    {
    ClientSize = new Size(SortForm.xWidth, SortForm.yHeight);
    SetStyle(ControlStyles.DoubleBuffer|ControlStyles.AllPaintingInWmPaint,
	     true);
    }

  protected internal void init(SortForm r)
    {
    root = r;
    t = new Thread(new ThreadStart(run));
    thr = "[" + t.Name + "] ";
    t.Start();
    }

  /*
   * load the sorting algorithm and run it, passing our copy of the current
   * pattern.
   */
  bool running = true;
  public void run()
    {
    running = true;
    while (running)
      {
      t.Suspend();
      if (!stopRequested)
	{
	try
	  {
	  sort(array);
	  if (!running)
	    break;
	  if (!stopRequested)
	    {
	    /*
	     * sort has returned and no stop was requested,
	     * so decrement active thread count
	     */
	    lock(root)		// lock parent
	      {
	      root.active_threads--;
	      if (root.active_threads <= 0)
		{
		root.resetButtons();
		}
	      }
	    }
	  }
	catch (Exception e) { Console.WriteLine(e); }
	pause();
	}
      }
    lock(root)		// lock parent
      {
      root.active_threads--;	// keep active thread count correct
      }
    }

  /*
   * stop the current sort
   */
  public void stopSort()
    {
    if (t != null)
      {
      stopRequested = true;
      }
    }
    
  public void startSort()
    {
    stopRequested = false;
    t.Resume();
    }

  public void stopThread()
    {
    stopRequested = true;
    running = false;
    t.Suspend();		// have to make sure its suspended
    t.Resume();			// so we can resume any suspended thread
    }

  /*
   * set the array pattern that this instance will use
   */
  public void setArray(int[] a)
    {
    array = (int[]) a.Clone();
    Invalidate();
    }


  /*
   * The high water mark.
   */
  int water_hi = -1;

  /*
   * The low water mark.
   */
  int water_lo = -1;

  /**
   * Pause a while, and draw the low&high water marks.
   */
  protected internal void pause(int whi, int wlo)
    {
    water_hi = whi;
    water_lo = wlo;

    Invalidate();
    /*
     * sleep this thread
     */
    if (running && root.sleep > 0)
	try { Thread.Sleep(root.sleep); } catch (ThreadInterruptedException) {}
    }

  /*
   * pause, but only draw hi water mark
   */
  protected internal void pause(int hi)
    {
    pause(hi, -1);
    }
  
  /*
   * pause
   */
  protected internal void pause()
    {
    pause(-1, -1);
    }

  protected override void OnPaint(PaintEventArgs e)
    {
    int y = Height;
    int x = Width;
    int xmid = x/2;

    Graphics g = e.Graphics;
    // erase old lines
    //    g.FillRectangle(grayBrush, 0, 0, x, y);
    g.FillRectangle(whiteBrush, 0, 0, x, y);

    if (array != null)
      {
      // draw the new lines
      y = y - 1;
      for (int i = array.Length; --i >= 0; y -= 2)
	{
	int dx = array[i]/2;
	g.DrawLine(blackPen, xmid-dx, y, xmid+dx, y);
	}

      if (water_hi >= 0)
	{
	y = water_hi*2 + 1;
	g.DrawLine(redPen, 0,y,x,y);
	}
      if (water_lo >= 0)
	{
	y = water_lo*2 + 1;
	g.DrawLine(bluePen, 0,y,x,y);
	}
      }
    }

  //  public new void Invalidate()
  //    {
  //    base.Invalidate();
  //    off_screen = null;
  //    }

  /**
   * This method will be called to
   * sort an array of integers.
   * This is overridden by the extenders
   */
  public abstract void sort(int[] a);
  public abstract string sortLang { get; }
  public abstract string sortAlg { get; }
  }
}
